
/**
 * A transaction consists of an account reference and an amount
 * A transaction must only be processed once.
 */
public abstract class Transaction
{
    // instance variables
    private int accountNumber;
    private float amount;
    private String description;
    private boolean processed;          // set to true after processing transaction

    /**
     * Constructor for objects of class Transaction
     * @param  acct   reference to the transaction account
     * @param  amt    the amount of the transaction (may be negative)
     * @param  desc   a description of the transaction
     */
    public Transaction(int acct, float amt, String desc)
    {
        // initialise instance variables
        accountNumber = acct;
        amount = amt;
        description = desc;
        processed = false;
    }

    /**
     * Accessor method for transaction amount
     *
     * @return     the transaction amount
     */
    public float getAmount()
    {
        return amount;
    }

    /**
     * Accessor method for transaction account number
     *
     * @return     the account number
     */
    public int getAccountNumber()
    {
        return accountNumber;
    }

    /**
     * Accessor method for transaction description
     *
     * @return     the transaction description
     */
    public String getDescription()
    {
        return description;
    }

    /**
     * Process this transaction.
     * Get the account from the ledger, call updateBalance
     * and set the processed flag if successful.
     *
     * @param  L   the ledger this transaction applies to
     */
    public void process(Ledger L)
    {
        Account acc = L.getAccount(accountNumber);
        
        if (acc != null)
        {
            processed = acc.updateBalance(this);
        }
    }

    /**
     * Process this transaction.
     * Get the account from the ledger, call updateBalance
     * and set the processed flag if successful.
     *
     * @param  L   the ledger this transaction applies to
     */
    public void process(Account acc)
    {
        if (acc != null)
        {
            processed = acc.updateBalance(this);
        }
    }

    /**
     * Creates a string representation of the transaction that can be printed
     *
     * @return     the string representation of the transaction
     */
    public String toString()
    {
        if (processed)
            return accountNumber + " $" + amount + " " + description;
        else
            return accountNumber + " $" + amount + " " + description + "(FAILED)";
    }
}
